
  
`timescale 1ns/1ps

`define C_S_AXI_DATA_WIDTH 32
`define C_S_AXI_ADDR_WIDTH 32
   
module spi_test
(
	 S_AXI_ACLK,
     S_AXI_ARESETN,
     S_AXI_AWADDR,
     S_AXI_AWVALID,
     S_AXI_AWREADY,
     S_AXI_WDATA,
     S_AXI_WSTRB,
     S_AXI_WVALID,
     S_AXI_WREADY,
     S_AXI_BRESP,
     S_AXI_BVALID,
     S_AXI_BREADY,
     S_AXI_ARADDR,
     S_AXI_ARVALID,
     S_AXI_ARREADY,
     S_AXI_RDATA,
     S_AXI_RRESP,
     S_AXI_RVALID,
     S_AXI_RREADY,
     err_flag,
     r_flag
);

 input                                 S_AXI_ACLK;
 input                                 S_AXI_ARESETN;
 output   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_AWADDR;
 output                                S_AXI_AWVALID;
 input                                 S_AXI_AWREADY;
 output   [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_WDATA;
 output   [`C_S_AXI_DATA_WIDTH/8-1:0]  S_AXI_WSTRB;
 output                                S_AXI_WVALID;
 input                                 S_AXI_WREADY;
 input  [1:0]                          S_AXI_BRESP;
 input                                 S_AXI_BVALID;
 output                                S_AXI_BREADY;
 output   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_ARADDR;
 output                                S_AXI_ARVALID;
 input                                 S_AXI_ARREADY;
 input  [`C_S_AXI_DATA_WIDTH-1:0]      S_AXI_RDATA;
 input  [1:0]                          S_AXI_RRESP;
 input                                 S_AXI_RVALID;
 output                                S_AXI_RREADY;
 output                                 err_flag;
 output                                 r_flag;
//////////////////////////////////////////////////////////////////////////
//	Internal Wires/Registers
 reg                                r_flag_reg = 0;
 reg                                err_flag_reg = 0;
 reg   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_AWADDR;
 reg                                S_AXI_AWVALID;

 reg   [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_WDATA;
 reg   [`C_S_AXI_DATA_WIDTH/8-1:0]  S_AXI_WSTRB;
 reg                                S_AXI_WVALID;
 
 reg                                 S_AXI_BREADY;
 reg   [`C_S_AXI_ADDR_WIDTH - 1:0]   S_AXI_ARADDR;
 reg                                 S_AXI_ARVALID;

 reg                                S_AXI_RREADY;
 
 reg								axi_awready_reg;
 reg								axi_arready_reg;
 reg								axi_rvalid_reg;
 
 wire  [`C_S_AXI_ADDR_WIDTH-1 : 0]        REG_RXDATA  = 32'h00;
 wire  [`C_S_AXI_ADDR_WIDTH-1 : 0]        REG_TXDATA  = 32'h04;
 wire  [`C_S_AXI_ADDR_WIDTH-1 : 0]        REG_STATUS  = 32'h08;
 wire  [`C_S_AXI_ADDR_WIDTH-1 : 0]        REG_CONTROL = 32'h0C;
 wire  [`C_S_AXI_ADDR_WIDTH-1 : 0]        REG_SSMASK  = 32'h10;

 reg[3:0]							wr_cntl;
 reg[3:0]							wr_reg;
 reg[3:0]							wr_index;
 reg[3:0]							rd_reg;
 reg[31:0]							rd_status;
 reg[3:0]							wr_reg1;
 reg[3:0]							rd_reg1;
 reg[31:0]                          rd_status1;
 reg[31:0]							rd_data;
///////////////////////////////////////////////////////////////////////////

always @(posedge S_AXI_ACLK)
begin
	if(~S_AXI_ARESETN)
		axi_awready_reg <= 1'b0;
	else
		axi_awready_reg <= S_AXI_AWREADY;
end

always @(posedge S_AXI_ACLK)
begin
	if(~S_AXI_ARESETN)
		axi_arready_reg <= 1'b0;
	else
		axi_arready_reg <= S_AXI_ARREADY;
end

always @(posedge S_AXI_ACLK)
begin
	if(~S_AXI_ARESETN)
		axi_rvalid_reg <= 1'b0;
	else
		axi_rvalid_reg <= S_AXI_RVALID;
end


	
always @(posedge S_AXI_ACLK)
begin
	if(~S_AXI_ARESETN)
	begin
		S_AXI_AWADDR <= {`C_S_AXI_ADDR_WIDTH{1'b0}};
		S_AXI_AWVALID <= 1'b0;
		S_AXI_WDATA <= {`C_S_AXI_DATA_WIDTH{1'b0}};
		S_AXI_WVALID <= 1'b0;
		S_AXI_WSTRB <= {`C_S_AXI_DATA_WIDTH/8 {1'b0}};
		
		S_AXI_ARVALID <= 1'b0;
		S_AXI_RREADY <= 1'b0;
		S_AXI_ARADDR <= {`C_S_AXI_ADDR_WIDTH{1'b0}};
		
		wr_index <= 0;
		wr_cntl <= 0;
		wr_reg <=0;
		rd_reg <=0;
		rd_status <=0;
		wr_reg1 <= 0;
		rd_reg1 <=0;
		rd_status1 <=0;
        err_flag_reg <= 0;
        r_flag_reg <= 0;
	end
	else
	begin
		if(wr_index==0)begin    //write ssmask
		    case(wr_cntl)
		                0:
		                    begin
		                        S_AXI_AWVALID <= 1'b1;
		                        S_AXI_WVALID <= 1'b1;
		                        S_AXI_AWADDR <= REG_SSMASK;
		                        S_AXI_WDATA <= {24'h0000_00,8'h01};	
		                        S_AXI_WSTRB <= 4'b0001;
		
		                        wr_cntl <=1;
		                    end
		                1:
		                    begin 
			                    if(S_AXI_AWREADY && ~axi_awready_reg)begin
				                    S_AXI_AWVALID <= 1'b0;
				                    S_AXI_WVALID <= 1'b0;
				                    S_AXI_AWADDR <= 0;
				                    S_AXI_WDATA <= 0;
				                    S_AXI_WSTRB <= 0;
			
				                    wr_cntl <=2;
			                    end
			                    else begin
				                    wr_cntl <=1;
			                    end	
		                    end
		                2:
		                    begin
			                    wr_index <= 1;
			                    wr_cntl <=0;
		                    end
		    endcase 
		end //if(wr_index==0)
        else if(wr_index==1)begin   //write control reg
		        case(wr_reg)
		                0:
		                    begin
		                        S_AXI_AWVALID <= 1'b1;
		                        S_AXI_WVALID <= 1'b1;
		                        S_AXI_AWADDR <= REG_CONTROL;
		                        //S_AXI_WDATA <= {24'h0000_00,8'h92};	//"5"
		                        S_AXI_WDATA <= {24'h0000_00,8'h8B};	//"d"
		                        S_AXI_WSTRB <= 4'b0001;
		
		                        wr_reg <=1;
		                    end
		                1:
		                    begin 
			                    if(S_AXI_AWREADY && ~axi_awready_reg)begin
				                    S_AXI_AWVALID <= 1'b0;
				                    S_AXI_WVALID <= 1'b0;
				                    S_AXI_AWADDR <= 0;
				                    S_AXI_WDATA <= 0;
				                    S_AXI_WSTRB <= 0;
			
				                    wr_reg <=2;
			                    end
		        	            else begin
				                    wr_reg <=1;
			                    end	
		                    end
		                2:
		                    begin
			                    wr_index <= 2;
			                    wr_reg <=0;
		                    end
		        endcase 
		end//if(wr_index==1)
        else if(wr_index==2)begin       //read status reg
			    case(rd_reg)
			            0:
			                begin
				                S_AXI_ARVALID <= 1'b1;
				                S_AXI_RREADY <= 1'b1;
				                S_AXI_ARADDR <= REG_STATUS;
				
				                rd_reg <= 1;
			                end
			            1:
			                begin
				                if(S_AXI_ARREADY && ~axi_arready_reg)begin
					                S_AXI_ARVALID <= 1'b0;
					
					                rd_reg <= 2; 
				                end
				                else begin
					                rd_reg <= 1;
				                end
			                end
			            2:
			                begin
				                if(S_AXI_RVALID && ~axi_rvalid_reg)begin
					                S_AXI_RREADY <= 1'b0;
					                rd_status <= S_AXI_RDATA;
					
					                rd_reg <= 3;					
				                end
				                else begin
					                rd_reg <= 2;
				                end
			                end	
			            3:
			                begin
					            if(rd_status[5]&&rd_status[4])begin
						            //wr_index <= 3;
						            wr_index <= 3;
						            rd_reg <= 0;
					            end
					            else
						            rd_reg <= 0;
				            end		
			        endcase
			end	//if(wr_index==2)
		else if(wr_index==3)begin   //write data
		        case(wr_reg)
		                0:
		                    begin
		                        S_AXI_AWVALID <= 1'b1;
		                        S_AXI_WVALID <= 1'b1;
		                        S_AXI_AWADDR <= REG_TXDATA;
		                        //S_AXI_WDATA <= {24'h0000_00,8'h92};	//"5"
		                        S_AXI_WDATA <= {24'h0000_00,8'h01};	//a1
		                        S_AXI_WSTRB <= 4'b0001;
		
		                        wr_reg <=1;
		                    end
		                1:
		                    begin 
			                    if(S_AXI_AWREADY && ~axi_awready_reg)begin
				                    S_AXI_AWVALID <= 1'b0;
				                    S_AXI_WVALID <= 1'b0;
				                    S_AXI_AWADDR <= 0;
				                    S_AXI_WDATA <= 0;
				                    S_AXI_WSTRB <= 0;
			
				                    wr_reg <=2;
			                    end
		        	            else begin
				                    wr_reg <=1;
			                    end	
		                    end
		                2:
		                    begin
			                    wr_index <= 4;
			                    wr_reg <=0;
		                    end
		        endcase 
		end//if(wr_index==3)
		else if(wr_index==4)begin       //read data
			    case(rd_reg)
			            0:
			                begin
				                S_AXI_ARVALID <= 1'b1;
				                S_AXI_RREADY <= 1'b1;
				                S_AXI_ARADDR <= REG_RXDATA;
				
				                rd_reg <= 1;
			                end
			            1:
			                begin
				                if(S_AXI_ARREADY && ~axi_arready_reg)begin
					                S_AXI_ARVALID <= 1'b0;
					
					                rd_reg <= 2; 
				                end
				                else begin
					                rd_reg <= 1;
				                end
			                end
			            2:
			                begin
				                if(S_AXI_RVALID && ~axi_rvalid_reg)begin
					                S_AXI_RREADY <= 1'b0;
					                rd_data <= S_AXI_RDATA;
                                    r_flag_reg <= 1;
                                    if((rd_data != 0) && (rd_data != 1))
                                        err_flag_reg <= 1;
					
					                rd_reg <= 0;
                                    wr_index <= 5;                    
				                end
				                else begin
					                rd_reg <= 2;
				                end
			                end	
			            		
			        endcase
			end	//if(wr_index==4)
        else if(wr_index==5)begin   //write control reg
		        case(wr_reg)
		                0:
		                    begin
		                        S_AXI_AWVALID <= 1'b1;
		                        S_AXI_WVALID <= 1'b1;
		                        S_AXI_AWADDR <= REG_CONTROL;
		                        //S_AXI_WDATA <= {24'h0000_00,8'h92};	//"5"
		                        S_AXI_WDATA <= {24'h0000_00,8'h00};	//"d"
		                        S_AXI_WSTRB <= 4'b0001;
		
		                        wr_reg <=1;
		                    end
		                1:
		                    begin 
			                    if(S_AXI_AWREADY && ~axi_awready_reg)begin
				                    S_AXI_AWVALID <= 1'b0;
				                    S_AXI_WVALID <= 1'b0;
				                    S_AXI_AWADDR <= 0;
				                    S_AXI_WDATA <= 0;
				                    S_AXI_WSTRB <= 0;
			
				                    wr_reg <=2;
			                    end
		        	            else begin
				                    wr_reg <=1;
			                    end	
		                    end
		                2:
		                    begin
			                    wr_index <= 0;
			                    wr_reg <=0;
		                    end
		        endcase 
		end//if(wr_index==5)
    	
	end 
end 
	
 assign err_flag = err_flag_reg;
 assign r_flag  = r_flag_reg;

endmodule
