  

`timescale 1ns/1ps

`define C_S_AXI_DATA_WIDTH 32
`define C_S_AXI_ADDR_WIDTH 32
   
module spi_test_top  
	 (
                    clk_50M,
                    rst_n ,
                    error_out,
                    r_data_flag
    );
        
    input			clk_50M;
    input			rst_n;
    output          error_out;
    output          r_data_flag;
            
////////////////////////////////////////////////////////////////   
 wire    [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_AWADDR;
 wire                                 S_AXI_AWVALID;
 wire                                 S_AXI_AWREADY;
 wire    [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_WDATA;
 wire    [`C_S_AXI_DATA_WIDTH/8-1:0]  S_AXI_WSTRB;
 wire                                 S_AXI_WVALID;
 wire                                 S_AXI_WREADY;
 wire    [1:0]                        S_AXI_BRESP;
 wire                                 S_AXI_BVALID;
 wire                                 S_AXI_BREADY;
 wire    [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_ARADDR;
 wire                                 S_AXI_ARVALID;
 wire                                 S_AXI_ARREADY;
 wire    [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_RDATA;
 wire    [1:0]                        S_AXI_RRESP;
 wire                                 S_AXI_RVALID;
 wire                                 S_AXI_RREADY;
 
 wire                                    MOSI_SLAVE;
 wire                                    MISO_SLAVE;
 wire                                    SS_N_SLAVE;
 wire                                    SCLK_SLAVE;

 wire                                    MOSI_MASTER;
 wire                                    SS_N_MASTER;
 wire                                    SCLK_MASTER;
 wire                                    MISO_MASTER;
//////////////////////////////////////////////////////////////////////////

    spi_test		u_spi_test
		(
		.S_AXI_ACLK         ( clk_50M                	),
        .S_AXI_ARESETN      ( rst_n            			),
        .S_AXI_AWADDR       ( S_AXI_AWADDR              ),
        .S_AXI_AWVALID      ( S_AXI_AWVALID             ),
        .S_AXI_AWREADY      ( S_AXI_AWREADY             ),
        .S_AXI_WDATA        ( S_AXI_WDATA               ),
        .S_AXI_WSTRB        ( S_AXI_WSTRB               ),
        .S_AXI_WVALID       ( S_AXI_WVALID              ),
        .S_AXI_WREADY       ( S_AXI_WREADY              ),
        .S_AXI_BRESP        ( S_AXI_BRESP               ),
        .S_AXI_BVALID       ( S_AXI_BVALID              ),
        .S_AXI_BREADY       ( S_AXI_BREADY              ),
        .S_AXI_ARADDR       ( S_AXI_ARADDR              ),
        .S_AXI_ARVALID      ( S_AXI_ARVALID             ),
        .S_AXI_ARREADY      ( S_AXI_ARREADY             ),
        .S_AXI_RDATA        ( S_AXI_RDATA               ),
        .S_AXI_RRESP        ( S_AXI_RRESP               ),
        .S_AXI_RVALID       ( S_AXI_RVALID              ),
        .S_AXI_RREADY       ( S_AXI_RREADY              ),
        .err_flag           ( error_out                 ),
        .r_flag             ( r_data_flag               )
		);

	 spi #(
        .SHIFT_DIRECTION    ( 0        					),
        .CLOCK_PHASE        ( 0               			),
        .CLOCK_POLARITY     ( 0          					),
        .CLOCK_SEL          ( 1                			),    // SCLK ( CLK_I / (2 * (CLOCK_SEL + 1)) ( CLK_I / 4
        .MASTER             ( 1                  		),
        .SLAVE_NUMBER       ( 1             				),    // maximum value ( 8 for current design
        .DATA_LENGTH        ( 8              			),    // changed from 32 to 8
        .DELAY_TIME         ( 2          			      ),
        .CLKCNT_WIDTH       ( 5       			     		),
        .INTERVAL_LENGTH    ( 2      				     	)
      ) 
    	u_spi (
        .S_AXI_ACLK         ( clk_50M               	 ),
        .S_AXI_ARESETN      ( rst_n             		 ),
        .S_AXI_AWADDR       ( S_AXI_AWADDR              ),
        .S_AXI_AWVALID      ( S_AXI_AWVALID             ),
        .S_AXI_AWREADY      ( S_AXI_AWREADY             ),
        .S_AXI_WDATA        ( S_AXI_WDATA               ),
        .S_AXI_WSTRB        ( S_AXI_WSTRB               ),
        .S_AXI_WVALID       ( S_AXI_WVALID              ),
        .S_AXI_WREADY       ( S_AXI_WREADY              ),
        .S_AXI_BRESP        ( S_AXI_BRESP               ),
        .S_AXI_BVALID       ( S_AXI_BVALID              ),
        .S_AXI_BREADY       ( S_AXI_BREADY              ),
        .S_AXI_ARADDR       ( S_AXI_ARADDR              ),
        .S_AXI_ARVALID      ( S_AXI_ARVALID             ),
        .S_AXI_ARREADY      ( S_AXI_ARREADY             ),
        .S_AXI_RDATA        ( S_AXI_RDATA               ),
        .S_AXI_RRESP        ( S_AXI_RRESP               ),
        .S_AXI_RVALID       ( S_AXI_RVALID              ),
        .S_AXI_RREADY       ( S_AXI_RREADY              ),
        .SPI_INT_O          (                           ),
        .MISO_MASTER        ( MISO_MASTER               ),
        .MOSI_MASTER        ( MOSI_MASTER               ),
        .SS_N_MASTER        ( SS_N_MASTER               ),
        .SCLK_MASTER        ( SCLK_MASTER               ),
        .MISO_SLAVE         ( MISO_SLAVE                ),
        .MOSI_SLAVE         ( MOSI_SLAVE                ),
        .SS_N_SLAVE         ( SS_N_SLAVE                ),
        .SCLK_SLAVE         ( SCLK_SLAVE                )
        );



    spi_slave_top  u_spi_slave_top
     (
         .SCLK              ( SCLK_MASTER               ),
         .MOSI              ( MOSI_MASTER               ),
         .MISO              ( MISO_MASTER               ),
         .SS                ( SS_N_MASTER               )

     );

          
endmodule
        
        
        
        
        
        
